import _noop from 'lodash-es/noop';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/* eslint react/no-multi-comp:0 */
import React, { Component } from 'react';
import TestUtils from 'react-addons-test-utils';
import { createSpy } from 'expect';
import { combineReducers as plainCombineReducers, createStore } from 'redux';
import { combineReducers as immutableCombineReducers } from 'redux-immutablejs';
import { Provider } from 'react-redux';

import createReducer from '../reducer';
import createReduxForm from '../reduxForm';
import createField from '../Field';
import createFieldArray from '../FieldArray';
import plain from '../structure/plain';
import plainExpectations from '../structure/plain/expectations';
import immutable from '../structure/immutable';
import immutableExpectations from '../structure/immutable/expectations';
import addExpectations from './addExpectations';
import { change } from '../actions';

var describeReduxForm = function describeReduxForm(name, structure, combineReducers, expect) {
  var fromJS = structure.fromJS;
  var getIn = structure.getIn;

  var reduxForm = createReduxForm(structure);
  var Field = createField(structure);
  var FieldArray = createFieldArray(structure);
  var reducer = createReducer(structure);

  describe(name, function () {
    var makeStore = function makeStore() {
      var initial = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      return createStore(combineReducers({ form: reducer }), fromJS({ form: initial }));
    };

    var propChecker = function propChecker(formState) {
      var renderSpy = arguments.length <= 1 || arguments[1] === undefined ? _noop : arguments[1];
      var config = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

      var store = makeStore({ testForm: formState });

      var Form = function (_Component) {
        _inherits(Form, _Component);

        function Form() {
          _classCallCheck(this, Form);

          return _possibleConstructorReturn(this, Object.getPrototypeOf(Form).apply(this, arguments));
        }

        _createClass(Form, [{
          key: 'render',
          value: function render() {
            renderSpy(this.props);
            return React.createElement(
              'div',
              null,
              React.createElement(Field, { name: 'foo', component: 'input' })
            );
          }
        }]);

        return Form;
      }(Component);

      var Decorated = reduxForm(_extends({ form: 'testForm' }, config))(Form);
      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));
      return TestUtils.findRenderedComponentWithType(dom, Form).props;
    };

    it('should return a decorator function', function () {
      expect(reduxForm).toBeA('function');
    });

    it('should render without error', function () {
      var store = makeStore();

      var Form = function (_Component2) {
        _inherits(Form, _Component2);

        function Form() {
          _classCallCheck(this, Form);

          return _possibleConstructorReturn(this, Object.getPrototypeOf(Form).apply(this, arguments));
        }

        _createClass(Form, [{
          key: 'render',
          value: function render() {
            return React.createElement('div', null);
          }
        }]);

        return Form;
      }(Component);

      expect(function () {
        var Decorated = reduxForm({ form: 'testForm' })(Form);
        TestUtils.renderIntoDocument(React.createElement(
          Provider,
          { store: store },
          React.createElement(Decorated, null)
        ));
      }).toNotThrow();
    });

    it('should provide dispatch prop', function () {
      expect(propChecker({}).dispatch).toExist().toBeA('function');
    });

    it('should provide dirty prop', function () {
      expect(propChecker({}).dirty).toBe(false);
      expect(propChecker({
        // no initial values
        values: {
          foo: 'bar'
        }
      }).dirty).toBe(true);
      expect(propChecker({
        initial: {
          foo: 'bar'
        },
        values: {
          foo: 'bar'
        }
      }).dirty).toBe(false);
      expect(propChecker({
        initial: {
          foo: 'bar'
        },
        values: {
          foo: 'baz'
        }
      }).dirty).toBe(true);
    });

    it('should provide pristine prop', function () {
      expect(propChecker({}).pristine).toBe(true);
      expect(propChecker({
        // no initial values
        values: {
          foo: 'bar'
        }
      }).pristine).toBe(false);
      expect(propChecker({
        initial: {
          foo: 'bar'
        },
        values: {
          foo: 'bar'
        }
      }).pristine).toBe(true);
      expect(propChecker({
        initial: {
          foo: 'bar'
        },
        values: {
          foo: 'baz'
        }
      }).pristine).toBe(false);
    });

    it('should provide valid prop', function () {
      expect(propChecker({}).valid).toBe(true);
      expect(propChecker({}, undefined, {
        validate: function validate() {
          return { foo: 'sync error' };
        }
      }).valid).toBe(false);
      expect(propChecker({
        asyncErrors: {
          foo: 'bar'
        }
      }).valid).toBe(false);
      expect(propChecker({
        asyncErrors: {
          nested: {
            myArrayField: [undefined, undefined]
          }
        }
      }).valid).toBe(true);
    });

    it('should provide invalid prop', function () {
      expect(propChecker({}).invalid).toBe(false);
      expect(propChecker({}, undefined, {
        validate: function validate() {
          return { foo: 'sync error' };
        }
      }).invalid).toBe(true);
      expect(propChecker({
        asyncErrors: {
          foo: 'bar'
        }
      }).invalid).toBe(true);
    });

    it('should provide submitting prop', function () {
      expect(propChecker({}).submitting).toBe(false);
      expect(propChecker({ submitting: true }).submitting).toBe(true);
      expect(propChecker({ submitting: false }).submitting).toBe(false);
    });

    it('should provide bound array action creators', function () {
      var arrayProp = propChecker({}).array;
      expect(arrayProp).toExist();
      expect(arrayProp.insert).toExist().toBeA('function');
      expect(arrayProp.pop).toExist().toBeA('function');
      expect(arrayProp.push).toExist().toBeA('function');
      expect(arrayProp.remove).toExist().toBeA('function');
      expect(arrayProp.shift).toExist().toBeA('function');
      expect(arrayProp.splice).toExist().toBeA('function');
      expect(arrayProp.swap).toExist().toBeA('function');
      expect(arrayProp.unshift).toExist().toBeA('function');
    });

    it('should not rerender unless form-wide props (except value!) change', function () {
      var spy = createSpy();

      var _propChecker = propChecker({}, spy, {
        validate: function validate(values) {
          var animal = getIn(values, 'animal');
          return animal && animal.length > 5 ? { animal: 'Too long' } : {};
        }
      });

      var dispatch = _propChecker.dispatch; // render 0

      expect(spy.calls.length).toBe(1);

      // simulate typing the word "giraffe"
      dispatch(change('testForm', 'animal', 'g')); // render 1 (now dirty)
      expect(spy.calls.length).toBe(2);

      dispatch(change('testForm', 'animal', 'gi')); // no render
      dispatch(change('testForm', 'animal', 'gir')); // no render
      dispatch(change('testForm', 'animal', 'gira')); // no render
      dispatch(change('testForm', 'animal', 'giraf')); // no render
      dispatch(change('testForm', 'animal', 'giraff')); // render 2 (invalid)
      expect(spy.calls.length).toBe(3);
      dispatch(change('testForm', 'animal', 'giraffe')); // no render

      dispatch(change('testForm', 'animal', '')); // render 3 (clean/valid)
      expect(spy.calls.length).toBe(4);

      expect(spy).toHaveBeenCalled();
      expect(spy.calls.length).toBe(4);

      expect(spy.calls[0].arguments[0].dirty).toBe(false);
      expect(spy.calls[0].arguments[0].invalid).toBe(false);
      expect(spy.calls[0].arguments[0].pristine).toBe(true);
      expect(spy.calls[0].arguments[0].valid).toBe(true);

      expect(spy.calls[1].arguments[0].dirty).toBe(true);
      expect(spy.calls[1].arguments[0].invalid).toBe(false);
      expect(spy.calls[1].arguments[0].pristine).toBe(false);
      expect(spy.calls[1].arguments[0].valid).toBe(true);

      expect(spy.calls[2].arguments[0].dirty).toBe(true);
      expect(spy.calls[2].arguments[0].invalid).toBe(true);
      expect(spy.calls[2].arguments[0].pristine).toBe(false);
      expect(spy.calls[2].arguments[0].valid).toBe(false);

      expect(spy.calls[3].arguments[0].dirty).toBe(false);
      expect(spy.calls[3].arguments[0].invalid).toBe(false);
      expect(spy.calls[3].arguments[0].pristine).toBe(true);
      expect(spy.calls[3].arguments[0].valid).toBe(true);
    });

    it('should initialize values with initialValues on first render', function () {
      var store = makeStore({});
      var inputRender = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();
      var formRender = createSpy();
      var initialValues = {
        deep: {
          foo: 'bar'
        }
      };

      var Form = function (_Component3) {
        _inherits(Form, _Component3);

        function Form() {
          _classCallCheck(this, Form);

          return _possibleConstructorReturn(this, Object.getPrototypeOf(Form).apply(this, arguments));
        }

        _createClass(Form, [{
          key: 'render',
          value: function render() {
            formRender(this.props);
            return React.createElement(
              'form',
              null,
              React.createElement(Field, { name: 'deep.foo', component: inputRender, type: 'text' })
            );
          }
        }]);

        return Form;
      }(Component);

      var Decorated = reduxForm({ form: 'testForm' })(Form);
      TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, { initialValues: initialValues })
      ));
      expect(store.getState()).toEqualMap({
        form: {
          testForm: {
            initial: initialValues,
            values: initialValues
          }
        }
      });
      expect(formRender).toHaveBeenCalled();
      expect(formRender.calls.length).toBe(1);
      var checkProps = function checkProps(props) {
        expect(props.pristine).toBe(true);
        expect(props.dirty).toBe(false);
        expect(props.initialized).toBe(false); // will be true on second render
      };
      checkProps(formRender.calls[0].arguments[0]);

      expect(inputRender).toHaveBeenCalled();
      expect(inputRender.calls.length).toBe(1);
      expect(inputRender.calls[0].arguments[0].pristine).toBe(true);
      expect(inputRender.calls[0].arguments[0].dirty).toBe(false);
      expect(inputRender.calls[0].arguments[0].value).toBe('bar');
    });

    it('should initialize with initialValues on later render', function () {
      var store = makeStore({});
      var inputRender = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();
      var formRender = createSpy();
      var initialValues = {
        deep: {
          foo: 'bar'
        }
      };

      var Form = function (_Component4) {
        _inherits(Form, _Component4);

        function Form() {
          _classCallCheck(this, Form);

          return _possibleConstructorReturn(this, Object.getPrototypeOf(Form).apply(this, arguments));
        }

        _createClass(Form, [{
          key: 'render',
          value: function render() {
            formRender(this.props);
            return React.createElement(
              'form',
              null,
              React.createElement(Field, { name: 'deep.foo', component: inputRender, type: 'text' })
            );
          }
        }]);

        return Form;
      }(Component);

      var Decorated = reduxForm({ form: 'testForm' })(Form);

      var Container = function (_Component5) {
        _inherits(Container, _Component5);

        function Container(props) {
          _classCallCheck(this, Container);

          var _this5 = _possibleConstructorReturn(this, Object.getPrototypeOf(Container).call(this, props));

          _this5.state = {};
          return _this5;
        }

        _createClass(Container, [{
          key: 'render',
          value: function render() {
            var _this6 = this;

            return React.createElement(
              'div',
              null,
              React.createElement(
                Provider,
                { store: store },
                React.createElement(Decorated, this.state)
              ),
              React.createElement(
                'button',
                { onClick: function onClick() {
                    return _this6.setState({ initialValues: initialValues });
                  } },
                'Init'
              )
            );
          }
        }]);

        return Container;
      }(Component);

      var dom = TestUtils.renderIntoDocument(React.createElement(Container, null));
      expect(store.getState()).toEqualMap({
        form: {}
      });
      expect(formRender).toHaveBeenCalled();
      expect(formRender.calls.length).toBe(1);
      var checkFormProps = function checkFormProps(props) {
        expect(props.pristine).toBe(true);
        expect(props.dirty).toBe(false);
        expect(props.initialized).toBe(false);
      };
      checkFormProps(formRender.calls[0].arguments[0]);

      expect(inputRender).toHaveBeenCalled();
      expect(inputRender.calls.length).toBe(1);
      var checkInputProps = function checkInputProps(props, value) {
        expect(props.pristine).toBe(true);
        expect(props.dirty).toBe(false);
        expect(props.value).toBe(value);
      };
      checkInputProps(inputRender.calls[0].arguments[0], '');

      // initialize
      var initButton = TestUtils.findRenderedDOMComponentWithTag(dom, 'button');
      TestUtils.Simulate.click(initButton);

      // check initialized state
      expect(store.getState()).toEqualMap({
        form: {
          testForm: {
            initial: initialValues,
            values: initialValues
          }
        }
      });

      // no need to rerender form on initialize
      expect(formRender.calls.length).toBe(1);

      // check rerendered input
      expect(inputRender.calls.length).toBe(2);
      checkInputProps(inputRender.calls[1].arguments[0], 'bar');
    });

    it('should destroy on unmount by default', function () {
      var store = makeStore({});
      var inputRender = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();
      var formRender = createSpy();

      var Form = function (_Component6) {
        _inherits(Form, _Component6);

        function Form() {
          _classCallCheck(this, Form);

          return _possibleConstructorReturn(this, Object.getPrototypeOf(Form).apply(this, arguments));
        }

        _createClass(Form, [{
          key: 'render',
          value: function render() {
            formRender(this.props);
            return React.createElement(
              'form',
              null,
              React.createElement(Field, { name: 'deep.foo', component: inputRender, type: 'text' })
            );
          }
        }]);

        return Form;
      }(Component);

      var Decorated = reduxForm({
        form: 'testForm'
      })(Form);

      var Container = function (_Component7) {
        _inherits(Container, _Component7);

        function Container(props) {
          _classCallCheck(this, Container);

          var _this8 = _possibleConstructorReturn(this, Object.getPrototypeOf(Container).call(this, props));

          _this8.state = { showForm: true };
          return _this8;
        }

        _createClass(Container, [{
          key: 'render',
          value: function render() {
            var _this9 = this;

            var showForm = this.state.showForm;

            return React.createElement(
              'div',
              null,
              React.createElement(
                Provider,
                { store: store },
                React.createElement(
                  'div',
                  null,
                  showForm && React.createElement(Decorated, this.state)
                )
              ),
              React.createElement(
                'button',
                { onClick: function onClick() {
                    return _this9.setState({ showForm: !showForm });
                  } },
                'Toggle'
              )
            );
          }
        }]);

        return Container;
      }(Component);

      var dom = TestUtils.renderIntoDocument(React.createElement(Container, null));
      expect(store.getState()).toEqualMap({
        form: {}
      }, 'Form data in Redux did not get destroyed');
      expect(formRender).toHaveBeenCalled();
      expect(formRender.calls.length).toBe(1);

      expect(inputRender).toHaveBeenCalled();
      expect(inputRender.calls.length).toBe(1);
      expect(inputRender.calls[0].arguments[0].value).toBe('');

      // change field
      inputRender.calls[0].arguments[0].onChange('bob');

      // form rerenders because now dirty
      expect(formRender.calls.length).toBe(2);

      // input now has value
      expect(inputRender.calls.length).toBe(2);
      expect(inputRender.calls[1].arguments[0].value).toBe('bob');

      // check state
      expect(store.getState()).toEqualMap({
        form: {
          testForm: {
            values: {
              deep: {
                foo: 'bob'
              }
            }
          }
        }
      });

      // unmount form
      var toggle = TestUtils.findRenderedDOMComponentWithTag(dom, 'button');
      TestUtils.Simulate.click(toggle);

      // check clean state
      expect(store.getState()).toEqualMap({
        form: {}
      });

      // form still not rendered again
      expect(formRender.calls.length).toBe(2);

      // toggle form back into existence
      TestUtils.Simulate.click(toggle);

      // form is back
      expect(formRender.calls.length).toBe(3);

      // input is back, but without value
      expect(inputRender.calls.length).toBe(3);
      expect(inputRender.calls[2].arguments[0].value).toBe('');
    });

    it('should not destroy on unmount if told not to', function () {
      var store = makeStore({});
      var inputRender = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();
      var formRender = createSpy();

      var Form = function (_Component8) {
        _inherits(Form, _Component8);

        function Form() {
          _classCallCheck(this, Form);

          return _possibleConstructorReturn(this, Object.getPrototypeOf(Form).apply(this, arguments));
        }

        _createClass(Form, [{
          key: 'render',
          value: function render() {
            formRender(this.props);
            return React.createElement(
              'form',
              null,
              React.createElement(Field, { name: 'deep.foo', component: inputRender, type: 'text' })
            );
          }
        }]);

        return Form;
      }(Component);

      var Decorated = reduxForm({
        form: 'testForm',
        destroyOnUnmount: false
      })(Form);

      var Container = function (_Component9) {
        _inherits(Container, _Component9);

        function Container(props) {
          _classCallCheck(this, Container);

          var _this11 = _possibleConstructorReturn(this, Object.getPrototypeOf(Container).call(this, props));

          _this11.state = { showForm: true };
          return _this11;
        }

        _createClass(Container, [{
          key: 'render',
          value: function render() {
            var _this12 = this;

            var showForm = this.state.showForm;

            return React.createElement(
              'div',
              null,
              React.createElement(
                Provider,
                { store: store },
                React.createElement(
                  'div',
                  null,
                  showForm && React.createElement(Decorated, this.state)
                )
              ),
              React.createElement(
                'button',
                { onClick: function onClick() {
                    return _this12.setState({ showForm: !showForm });
                  } },
                'Toggle'
              )
            );
          }
        }]);

        return Container;
      }(Component);

      var dom = TestUtils.renderIntoDocument(React.createElement(Container, null));
      expect(store.getState()).toEqualMap({
        form: {}
      }, 'Form data in Redux did not get destroyed');
      expect(formRender).toHaveBeenCalled();
      expect(formRender.calls.length).toBe(1);

      expect(inputRender).toHaveBeenCalled();
      expect(inputRender.calls.length).toBe(1);
      expect(inputRender.calls[0].arguments[0].value).toBe('');

      // change field
      inputRender.calls[0].arguments[0].onChange('bob');

      // form rerenders because now dirty
      expect(formRender.calls.length).toBe(2);

      // input now has value
      expect(inputRender.calls.length).toBe(2);
      expect(inputRender.calls[1].arguments[0].value).toBe('bob');

      // check state
      expect(store.getState()).toEqualMap({
        form: {
          testForm: {
            values: {
              deep: {
                foo: 'bob'
              }
            }
          }
        }
      });

      // unmount form
      var toggle = TestUtils.findRenderedDOMComponentWithTag(dom, 'button');
      TestUtils.Simulate.click(toggle);

      // check state not destroyed
      expect(store.getState()).toEqualMap({
        form: {
          testForm: {
            values: {
              deep: {
                foo: 'bob'
              }
            }
          }
        }
      });

      // form still not rendered again
      expect(formRender.calls.length).toBe(2);

      // toggle form back into existence
      TestUtils.Simulate.click(toggle);

      // form is back
      expect(formRender.calls.length).toBe(3);

      // input is back, with its old value
      expect(inputRender.calls.length).toBe(3);
      expect(inputRender.calls[2].arguments[0].value).toBe('bob');
    });

    it('should keep a list of registered fields', function () {
      var store = makeStore({});
      var noopRender = function noopRender() {
        return React.createElement('div', null);
      };

      var Form = function (_Component10) {
        _inherits(Form, _Component10);

        function Form() {
          _classCallCheck(this, Form);

          var _this13 = _possibleConstructorReturn(this, Object.getPrototypeOf(Form).call(this));

          _this13.state = { showBar: false };
          return _this13;
        }

        _createClass(Form, [{
          key: 'render',
          value: function render() {
            var _this14 = this;

            var showBar = this.state.showBar;

            return React.createElement(
              'form',
              null,
              !showBar && React.createElement(Field, { name: 'foo', component: 'input', type: 'text' }),
              !showBar && React.createElement(FieldArray, { name: 'fooArray', component: noopRender, type: 'text' }),
              showBar && React.createElement(Field, { name: 'bar', component: 'input', type: 'text' }),
              showBar && React.createElement(FieldArray, { name: 'barArray', component: noopRender, type: 'text' }),
              React.createElement(
                'button',
                { onClick: function onClick() {
                    return _this14.setState({ showBar: true });
                  } },
                'Show Bar'
              )
            );
          }
        }]);

        return Form;
      }(Component);

      var Decorated = reduxForm({ form: 'testForm' })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      var stub = TestUtils.findRenderedComponentWithType(dom, Decorated);
      expect(stub.fieldList).toEqual(['foo', 'fooArray']);

      // switch fields
      var button = TestUtils.findRenderedDOMComponentWithTag(dom, 'button');
      TestUtils.Simulate.click(button);

      expect(stub.fieldList).toEqual(['bar', 'barArray']);
    });

    it('should provide valid/invalid/values getters', function () {
      var store = makeStore({});
      var input = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();

      var Form = function Form() {
        return React.createElement(
          'form',
          null,
          React.createElement(Field, { name: 'bar', component: input, type: 'text' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm',
        validate: function validate(values) {
          return getIn(values, 'bar') ? {} : { bar: 'Required' };
        }
      })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      var stub = TestUtils.findRenderedComponentWithType(dom, Decorated);

      // invalid because no value for 'bar' field
      expect(stub.valid).toBe(false);
      expect(stub.invalid).toBe(true);
      expect(stub.values).toEqualMap({});

      // set value for 'bar' field
      input.calls[0].arguments[0].onChange('foo');

      // valid because we have a value for 'bar' field
      expect(stub.valid).toBe(true);
      expect(stub.invalid).toBe(false);
      expect(stub.values).toEqualMap({ bar: 'foo' });
    });

    it('should submit when submit() called and onSubmit provided as config param', function () {
      var store = makeStore({
        testForm: {
          values: {
            bar: 'foo'
          }
        }
      });
      var input = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();

      var Form = function Form() {
        return React.createElement(
          'form',
          null,
          React.createElement(Field, { name: 'bar', component: input, type: 'text' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm',
        onSubmit: function onSubmit(values) {
          expect(values).toEqualMap({ bar: 'foo' });
        }
      })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      var stub = TestUtils.findRenderedComponentWithType(dom, Decorated);

      expect(input).toHaveBeenCalled();
      expect(input.calls[0].arguments[0].value).toBe('foo');

      expect(stub.submit).toBeA('function');
      stub.submit();
    });

    it('should submit when "submit" button is clicked and handleSubmit provided function', function () {
      var store = makeStore({
        testForm: {
          values: {
            bar: 'foo'
          }
        }
      });
      var submit = createSpy();

      var Form = function Form(_ref) {
        var handleSubmit = _ref.handleSubmit;
        return React.createElement(
          'form',
          { onSubmit: handleSubmit(submit) },
          React.createElement(Field, { name: 'bar', component: 'textarea' }),
          React.createElement('input', { type: 'submit', value: 'Submit' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm'
      })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      var form = TestUtils.findRenderedDOMComponentWithTag(dom, 'form');

      expect(submit).toNotHaveBeenCalled();

      TestUtils.Simulate.submit(form);

      expect(submit).toHaveBeenCalled();
    });

    it('should be fine if form is not yet in Redux store', function () {
      var store = makeStore({
        anotherForm: {
          values: {
            bar: 'foo'
          }
        }
      });
      var input = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();

      var Form = function Form() {
        return React.createElement(
          'form',
          null,
          React.createElement(Field, { name: 'foo', component: input, type: 'text' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm'
      })(Form);

      TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      expect(input).toHaveBeenCalled();
      expect(input.calls[0].arguments[0].value).toBe('');
    });

    it('should be fine if getFormState returns nothing', function () {
      var store = makeStore({});
      var input = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();

      var Form = function Form() {
        return React.createElement(
          'form',
          null,
          React.createElement(Field, { name: 'foo', component: input, type: 'text' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm',
        getFormState: function getFormState() {
          return undefined;
        }
      })(Form);

      TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      expect(input).toHaveBeenCalled();
      expect(input.calls[0].arguments[0].value).toBe('');
    });

    it('should throw an error when no onSubmit is specified', function () {
      var store = makeStore({
        testForm: {
          values: {
            bar: 'foo'
          }
        }
      });

      var Form = function Form() {
        return React.createElement(
          'form',
          null,
          React.createElement(Field, { name: 'bar', component: 'input', type: 'text' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm'
      })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      var stub = TestUtils.findRenderedComponentWithType(dom, Decorated);
      expect(function () {
        return stub.submit();
      }).toThrow(/onSubmit function or pass onSubmit as a prop/);
    });

    it('should submit (with async validation) when submit() called', function () {
      var store = makeStore({
        testForm: {
          values: {
            bar: 'foo'
          }
        }
      });
      var input = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();
      var asyncValidate = createSpy(function () {
        return Promise.resolve();
      }).andCallThrough();

      var Form = function Form() {
        return React.createElement(
          'form',
          null,
          React.createElement(Field, { name: 'bar', component: input, type: 'text' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm',
        asyncValidate: asyncValidate,
        onSubmit: function onSubmit(values) {
          expect(values).toEqualMap({ bar: 'foo' });
        }
      })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      var stub = TestUtils.findRenderedComponentWithType(dom, Decorated);

      expect(input).toHaveBeenCalled();
      expect(input.calls[0].arguments[0].value).toBe('foo');

      expect(asyncValidate).toNotHaveBeenCalled();

      expect(stub.submit).toBeA('function');
      stub.submit();

      expect(asyncValidate).toHaveBeenCalled();
      expect(asyncValidate.calls[0].arguments[0]).toEqualMap({ bar: 'foo' });
    });

    it('should not call async validation more than once if submit is clicked fast', function (done) {
      var store = makeStore({
        testForm: {
          values: {
            bar: 'foo'
          }
        }
      });
      var input = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();
      var asyncValidate = createSpy(function () {
        return new Promise(function (resolve) {
          return setTimeout(resolve, 100);
        });
      }).andCallThrough();

      var Form = function Form(_ref2) {
        var handleSubmit = _ref2.handleSubmit;
        return React.createElement(
          'form',
          { onSubmit: handleSubmit },
          React.createElement(Field, { name: 'bar', component: input, type: 'text' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm',
        asyncValidate: asyncValidate,
        onSubmit: function onSubmit(values) {
          expect(values).toEqualMap({ bar: 'foo' });
          done();
        }
      })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      var form = TestUtils.findRenderedDOMComponentWithTag(dom, 'form');

      expect(input).toHaveBeenCalled();
      expect(input.calls[0].arguments[0].value).toBe('foo');

      expect(asyncValidate).toNotHaveBeenCalled();

      TestUtils.Simulate.submit(form);
      TestUtils.Simulate.submit(form);
      TestUtils.Simulate.submit(form);
      TestUtils.Simulate.submit(form);
      TestUtils.Simulate.submit(form);

      expect(asyncValidate).toHaveBeenCalled();
      expect(asyncValidate.calls.length).toBe(1);
      expect(asyncValidate.calls[0].arguments[0]).toEqualMap({ bar: 'foo' });
    });

    it('should reset when reset() called', function () {
      var store = makeStore({});
      var input = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();

      var Form = function Form() {
        return React.createElement(
          'form',
          null,
          React.createElement(Field, { name: 'bar', component: input, type: 'text' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm',
        initialValues: { bar: 'initialBar' }
      })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));

      var stub = TestUtils.findRenderedComponentWithType(dom, Decorated);

      expect(input).toHaveBeenCalled();

      expect(input.calls[0].arguments[0].value).toBe('initialBar');

      input.calls[0].arguments[0].onChange('newBar');

      expect(input.calls[1].arguments[0].value).toBe('newBar');

      expect(stub.reset).toBeA('function');
      stub.reset();

      expect(input.calls[2].arguments[0].value).toBe('initialBar');
    });

    it('should rerender form, but not fields, when non-redux-form props change', function () {
      var store = makeStore({});
      var inputRender = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();
      var formRender = createSpy();

      var Form = function (_Component11) {
        _inherits(Form, _Component11);

        function Form() {
          _classCallCheck(this, Form);

          return _possibleConstructorReturn(this, Object.getPrototypeOf(Form).apply(this, arguments));
        }

        _createClass(Form, [{
          key: 'render',
          value: function render() {
            formRender(this.props);
            return React.createElement(
              'form',
              null,
              React.createElement(Field, { name: 'deep.foo', component: inputRender, type: 'text' })
            );
          }
        }]);

        return Form;
      }(Component);

      var Decorated = reduxForm({ form: 'testForm' })(Form);

      var Container = function (_Component12) {
        _inherits(Container, _Component12);

        function Container(props) {
          _classCallCheck(this, Container);

          var _this16 = _possibleConstructorReturn(this, Object.getPrototypeOf(Container).call(this, props));

          _this16.state = {};
          return _this16;
        }

        _createClass(Container, [{
          key: 'render',
          value: function render() {
            var _this17 = this;

            return React.createElement(
              'div',
              null,
              React.createElement(
                Provider,
                { store: store },
                React.createElement(Decorated, this.state)
              ),
              React.createElement(
                'button',
                { onClick: function onClick() {
                    return _this17.setState({ someOtherProp: 42 });
                  } },
                'Init'
              )
            );
          }
        }]);

        return Container;
      }(Component);

      var dom = TestUtils.renderIntoDocument(React.createElement(Container, null));
      expect(store.getState()).toEqualMap({
        form: {}
      });
      expect(formRender).toHaveBeenCalled();
      expect(formRender.calls.length).toBe(1);
      expect(formRender.calls[0].arguments[0].someOtherProp).toNotExist();

      expect(inputRender).toHaveBeenCalled();
      expect(inputRender.calls.length).toBe(1);

      // initialize
      var initButton = TestUtils.findRenderedDOMComponentWithTag(dom, 'button');
      TestUtils.Simulate.click(initButton);

      // rerender form on prop change
      expect(formRender.calls.length).toBe(2);
      expect(formRender.calls[1].arguments[0].someOtherProp).toExist().toBe(42);

      // no need to rerender input
      expect(inputRender.calls.length).toBe(1);
    });

    it('should call async on blur of async blur field', function (done) {
      var store = makeStore({});
      var inputRender = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();
      var formRender = createSpy();
      var asyncErrors = {
        deep: {
          foo: 'async error'
        }
      };
      var asyncValidate = createSpy().andReturn(Promise.reject(asyncErrors));

      var Form = function (_Component13) {
        _inherits(Form, _Component13);

        function Form() {
          _classCallCheck(this, Form);

          return _possibleConstructorReturn(this, Object.getPrototypeOf(Form).apply(this, arguments));
        }

        _createClass(Form, [{
          key: 'render',
          value: function render() {
            formRender(this.props);
            return React.createElement(
              'form',
              null,
              React.createElement(Field, { name: 'deep.foo', component: inputRender, type: 'text' })
            );
          }
        }]);

        return Form;
      }(Component);

      var Decorated = reduxForm({
        form: 'testForm',
        asyncValidate: asyncValidate,
        asyncBlurFields: ['deep.foo']
      })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));
      expect(store.getState()).toEqualMap({
        form: {}
      });
      expect(formRender).toHaveBeenCalled();
      expect(formRender.calls.length).toBe(1);

      expect(asyncValidate).toNotHaveBeenCalled();

      expect(inputRender).toHaveBeenCalled();
      expect(inputRender.calls.length).toBe(1);
      expect(inputRender.calls[0].arguments[0].pristine).toBe(true);
      expect(inputRender.calls[0].arguments[0].value).toBe('');
      expect(inputRender.calls[0].arguments[0].valid).toBe(true);
      expect(inputRender.calls[0].arguments[0].error).toBe(undefined);

      var inputElement = TestUtils.findRenderedDOMComponentWithTag(dom, 'input');
      TestUtils.Simulate.change(inputElement, { target: { value: 'bar' } });

      expect(store.getState()).toEqualMap({
        form: {
          testForm: {
            values: {
              deep: {
                foo: 'bar'
              }
            }
          }
        }
      });
      expect(formRender.calls.length).toBe(2); // rerendered because pristine -> dirty

      expect(asyncValidate).toNotHaveBeenCalled(); // not yet

      expect(inputRender.calls.length).toBe(2); // input rerendered
      expect(inputRender.calls[1].arguments[0].pristine).toBe(false);
      expect(inputRender.calls[1].arguments[0].value).toBe('bar');
      expect(inputRender.calls[1].arguments[0].valid).toBe(true);
      expect(inputRender.calls[1].arguments[0].error).toBe(undefined);

      TestUtils.Simulate.blur(inputElement, { target: { value: 'bar' } });

      setTimeout(function () {
        expect(store.getState()).toEqualMap({
          form: {
            testForm: {
              anyTouched: true,
              values: {
                deep: {
                  foo: 'bar'
                }
              },
              fields: {
                deep: {
                  foo: {
                    touched: true
                  }
                }
              },
              asyncErrors: asyncErrors
            }
          }
        });
        // rerender form twice because of async validation start and again for valid -> invalid
        expect(formRender.calls.length).toBe(4);

        expect(asyncValidate).toHaveBeenCalled();
        expect(asyncValidate.calls[0].arguments[0]).toEqualMap({ deep: { foo: 'bar' } });

        // input rerendered twice, at start and end of async validation
        expect(inputRender.calls.length).toBe(4);
        expect(inputRender.calls[3].arguments[0].pristine).toBe(false);
        expect(inputRender.calls[3].arguments[0].value).toBe('bar');
        expect(inputRender.calls[3].arguments[0].valid).toBe(false);
        expect(inputRender.calls[3].arguments[0].error).toBe('async error');
        done();
      });
    });

    it('should not call async validate if shouldAsyncValidate returns false', function () {
      var store = makeStore({});
      var inputRender = createSpy(function (props) {
        return React.createElement('input', props);
      }).andCallThrough();
      var asyncValidate = createSpy().andReturn(Promise.reject({ foo: 'bad user!' }));
      var shouldAsyncValidate = createSpy().andReturn(false);

      var Form = function Form() {
        return React.createElement(
          'form',
          null,
          React.createElement(Field, { name: 'foo', component: inputRender, type: 'text' })
        );
      };

      var Decorated = reduxForm({
        form: 'testForm',
        asyncValidate: asyncValidate,
        asyncBlurFields: ['foo'],
        shouldAsyncValidate: shouldAsyncValidate
      })(Form);

      var dom = TestUtils.renderIntoDocument(React.createElement(
        Provider,
        { store: store },
        React.createElement(Decorated, null)
      ));
      expect(store.getState()).toEqualMap({
        form: {}
      });

      expect(asyncValidate).toNotHaveBeenCalled();

      var inputElement = TestUtils.findRenderedDOMComponentWithTag(dom, 'input');
      TestUtils.Simulate.change(inputElement, { target: { value: 'bar' } });

      expect(shouldAsyncValidate).toNotHaveBeenCalled();

      TestUtils.Simulate.blur(inputElement, { target: { value: 'bar' } });

      expect(shouldAsyncValidate).toHaveBeenCalled();

      expect(asyncValidate).toNotHaveBeenCalled();
    });
  });
};

describeReduxForm('reduxForm.plain', plain, plainCombineReducers, addExpectations(plainExpectations));
describeReduxForm('reduxForm.immutable', immutable, immutableCombineReducers, addExpectations(immutableExpectations));